from baelfire.task import Task
from baelfire.dependencies import ParentFileChanged


class Virtualenv(Task):
    name = 'Create virtualenv'
    help = 'Generates virtual envoritment'

    def get_output_file(self):
        return self.paths['virtualenv_path']

    def make(self):
        self.command(['virtualenv -p python%s %s' % (
            self.python_version(),
            self.paths['virtualenv_path'])]
        )

    def python_version(self):
        if self.kwargs.get('python_version', False):
            return self.kwargs['python_version'][0]
        else:
            return self.settings['python_version']


class VirtualenvTask(Task):

    def python(self, command, *args, **kwargs):
        command = self.paths['VEpython'] + ' ' + command
        return self.command([command], *args, **kwargs)

    def pip(self, command, *args, **kwargs):
        command = self.paths['VEpip'] + ' ' + command
        return self.command([command], *args, **kwargs)


class Develop(VirtualenvTask):
    name = 'Setup develop'
    help = 'Run setup.py develop with virtualenv'

    def get_output_file(self):
        return self.paths['develop_flag']

    def generate_dependencies(self):
        self.add_dependecy(
            ParentFileChanged(
                self.task(
                    'bael.project.tasks:SetupPy')))

    def generate_links(self):
        self.add_link('bael.project.tasks:Create')
        self.add_link('bael.project.virtualenv:Virtualenv')

    def make(self):
        self.python('%(setuppy)s develop' % self.paths)
        self.touchme()



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
]

if __name__ == '__main__':
    setup(name='{{settings["project_name"]}}',
          version='0.1',
          packages=find_packages('src'),
          package_dir={'': 'src'},
          install_requires=install_requires,
          include_package_data=True,
          )



from os import mkdir, path

from baelfire.task import Task
from baelfire.template import TemplateTask
from baelfire.dependencies import (
    AlwaysRebuild,
    FileDoesNotExists,
)


class GatherData(Task):
    hide = True
    name = 'Gather data'

    def make(self):
        self.ask_for_setting('project_name', 'Project name')
        self.settings['package_name'] = self.settings['project_name'].lower()
        self.paths['project_home'] = [
            '%(src)s',
            self.settings['package_name']]

    def generate_dependencies(self):
        self.add_dependecy(AlwaysRebuild())


class SetupPy(TemplateTask):
    hide = True
    name = 'Creating setup.py file'

    def __init__(self, *args, **kwargs):
        kwargs['check_template'] = False
        super().__init__(*args, **kwargs)

    def get_output_file(self):
        return self.paths['setuppy']

    def get_template_path(self):
        return self.paths['setuppy']

    def pre_invoked_tasks(self):
        self.invoke_task(GatherData)


class Directories(Task):
    hide = True
    name = 'Creating directories'

    directorie_names = [
        'src',
        'flags',
        'project_home',
    ]

    @property
    def directories(self):
        for name in self.directorie_names:
            yield self.paths[name]

    def generate_dependencies(self):
        for directory in self.directories:
            self.add_dependecy(FileDoesNotExists(directory))

    def make(self):
        for directory in self.directories:
            if not path.exists(directory):
                mkdir(directory)

    def pre_invoked_tasks(self):
        self.invoke_task(GatherData)


class Inits(Task):
    hide = True
    name = 'Creating __init__ files'

    def paths(self):
        url = Directories
        for directory in self.task(url).directories:
            yield path.join(directory, '__init__.py')

    def generate_dependencies(self):
        for file_path in self.paths():
            self.add_dependecy(FileDoesNotExists(file_path))

    def generate_links(self):
        self.add_link(Directories)

    def make(self):
        for file_path in self.paths():
            self.touch(file_path)


class Create(Task):

    help = 'Creates sample python repository'

    def generate_dependencies(self):
        pass

    def generate_links(self):
        self.add_link(SetupPy)
        self.add_link(Inits)



from baelfire.task import Task
from baelfire.template import TemplateTask


class Ignore(TemplateTask):
    hide = True
    name = 'Creating .gitignore file'
    path = '/git/ignore'

    def get_output_file(self):
        return '.gitignore'

    def get_template_path(self):
        return '.gitignore.tpl'


class Init(Task):
    hide = True
    name = 'Initializing git repository'
    path = '/git/init'

    def get_output_file(self):
        return '.git'

    def generate_links(self):
        self.add_link('bael.project.git:Ignore')

    def make(self):
        self.command(['git init'])


class Commit(Task):
    name = 'Git initial commit'
    path = '/git'
    help = 'Creates sample python repository with git'

    def get_output_file(self):
        return '.git.flag'

    def generate_links(self):
        self.add_link('bael.project.tasks:Create')
        self.add_link('bael.project.git:Init')

    def make(self):
        self.command(['git add .'])
        self.command(['git commit -a -m "Initial commit by PyProject."'])
        self.touchme()


class Develop(Task):
    name = 'Develop with git'
    path = '/develop/git'
    help = 'Run setup.py develop with virtualenv with git repository'

    def generate_links(self):
        self.add_link('bael.project.virtualenv:Develop')
        self.add_link(Commit)

    def make(self):
        pass



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'baelfire==0.2',
    'virtualenv',
]

if __name__ == '__main__':
    setup(name='bael.project',
          version='0.1',
          packages=find_packages('src'),
          package_dir={'': 'src'},
          namespace_packages=['bael'],
          install_requires=install_requires,
          include_package_data=True,
          zip_safe=False,
          package_data={
              '': ['.gitignore.tpl'],
          },
          entry_points="""\
              [console_scripts]
                  pyproject = bael.project.recipe:run
          """,
          )



from baelfire.recipe import Recipe
from baelfire.application.application import Application

from .tasks import (
    Create,
    SetupPy,
    GatherData,
    Directories,
    Inits,
)

from .git import (
    Ignore,
    Init,
    Commit,
    Develop as GitDevelop,
)

from .virtualenv import Virtualenv, Develop


class ProjectRecipe(Recipe):

    def __init__(self, python_version='3.4'):
        self.python_version = python_version
        super().__init__()

    def final_settings(self):
        self.settings['python_version'] = self.python_version

    def create_settings(self):
        self.paths['virtualenv_path'] = 'venv'
        self.paths['setuppy'] = 'setup.py'
        self.paths['src'] = 'src'
        self.paths['project_home'] = ['%(src)s']
        self.paths['flags'] = 'flags'
        self.paths['develop_flag'] = ['%(flags)s', 'develop.flag']

        self.paths['VEbin'] = ['%(virtualenv_path)s', 'bin']
        self.paths['VEpython'] = ['%(VEbin)s', 'python']
        self.paths['VEpip'] = ['%(VEbin)s', 'pip']

    def gather_recipes(self):
        self.add_task(Create)
        self.add_task(SetupPy)
        self.add_task(GatherData)
        self.add_task(Directories)
        self.add_task(Inits)
        self.add_task(Ignore)
        self.add_task(Init)
        self.add_task(Commit)
        self.add_task(Virtualenv)
        self.add_task(Develop)
        self.add_task(GitDevelop)


def run():
    Application(recipe=ProjectRecipe())()
